package com.jguerson.tests;

import java.io.IOException;
import java.util.ArrayList;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.ocl.examples.pivot.Constraint;
import org.eclipse.ocl.examples.pivot.OCL;
import org.eclipse.ocl.examples.pivot.ParserException;
import org.eclipse.ocl.examples.pivot.manager.MetaModelManager;
import org.eclipse.ocl.examples.pivot.model.OCLstdlib;
import org.eclipse.ocl.examples.pivot.utilities.PivotEnvironmentFactory;

public class PivotOCLTest {
	
	private PivotEnvironmentFactory pivotEnvFactory;
	private OCL ocl;
	private String oclPath;	
	private ArrayList<Constraint> constraintlist = new ArrayList<Constraint>();
	private EPackage ecoreRoot;
	private MetaModelManager metamodelManager;
	
	private void setupOCL(ResourceSet resourceSet)
	{	
		OCLstdlib.install();		
		org.eclipse.ocl.examples.xtext.completeocl.CompleteOCLStandaloneSetup.doSetup();
    	org.eclipse.ocl.examples.xtext.essentialocl.EssentialOCLStandaloneSetup.doSetup();    	
    	org.eclipse.ocl.examples.xtext.oclstdlib.OCLstdlibStandaloneSetup.doSetup();
    	
		OCL.initialize(resourceSet);
	}
	
	public static EPackage readEcore (String ecorePath)
	{
		URI ecoreURI = URI.createFileURI(ecorePath);	
		
		ResourceSet ecoreResourceSet = new ResourceSetImpl();
				
		ecoreResourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("ecore", new EcoreResourceFactoryImpl());
		ecoreResourceSet.getPackageRegistry().put(EcorePackage.eNS_URI, EcorePackage.eINSTANCE);
		    	
	    Resource ecoreResource = ecoreResourceSet.getResource(ecoreURI,true);
	    EPackage ecoremodel = (EPackage) ecoreResource.getContents().get(0);
	    ecoreResource.getResourceSet().getPackageRegistry().put(null,ecoremodel);
	    
	    return ecoremodel;
	}

	/** Constructor 
	 * @throws ParserException */
	public PivotOCLTest (String oclAbsolutePath, String refAbsolutePath) throws IOException, ParserException
	{			
		oclPath = oclAbsolutePath;

		ecoreRoot = readEcore(refAbsolutePath);
		
		setupOCL(ecoreRoot.eResource().getResourceSet());
		
		metamodelManager = new MetaModelManager();
		pivotEnvFactory = new PivotEnvironmentFactory(ecoreRoot.eResource().getResourceSet().getPackageRegistry(), metamodelManager);
	    ocl = OCL.newInstance(pivotEnvFactory);		

	    URI oclURI = URI.createFileURI(oclPath);
	    
		// parse the contents as an OCL document
		Resource pivotResource = ocl.parse(oclURI);
	    for (TreeIterator<EObject> tit = pivotResource.getAllContents(); tit.hasNext(); ) 
	    {
	    	EObject next = tit.next();
	    	if (next instanceof Constraint) 
	    	{
		        Constraint constraint = (Constraint)next;				
				System.out.println("ETarget = "+constraint.getETarget());				
	    	}
	    }
   	}	    

	public MetaModelManager getMetamodelManager() {
		return metamodelManager;
	}

	public ArrayList<Constraint> getConstraints()
	{
		return constraintlist;
	}

	public EPackage getEcoreRoot ()
	{
		return ecoreRoot;
	}
	
	public OCL getOCL()
	{
		return ocl;
	}
	
	/** Run a Test */
	public static void main (String[] args)
	{				
		String oclPath = "model/project.ocl";				
		String refPath = "model/project.ecore";
		
		try {
			
			new PivotOCLTest(oclPath,refPath);
			System.out.println("OCL parsed.");
			
		} catch (IOException e) {
			e.printStackTrace();
			
		} catch (ParserException e) {
			e.printStackTrace();
		}
		
		
	}
}
